@@ -20,7 +20,7 @@
 #include <math.h>
 #include "crinkle.h"
 
-char calcalt_Id[] = "$Id: calcalt.c,v 2.12 1996/09/13 10:07:46 spb Exp spb $";
+char calcalt_Id[] = "$Id: calcalt.c,v 2.17 2001/03/30 12:35:32 spb Exp $";
 
 #ifdef DEBUG
 #define DB(A,B) dump_pipeline(A,B)
@@ -29,9 +29,13 @@ char calcalt_Id[] = "$Id: calcalt.c,v 2.12 1996/09/13 10:07:46 spb Exp spb $";
 #endif
 
 
-/*{{{  Strip *make_strip(Fold *f) */
+/* {{{   Strip *make_strip(Fold *f) */
+#ifdef ANSI
+  Strip *make_strip(Fold *f) 
+#else
 Strip *make_strip (f)
 Fold *f;
+#endif
 {
   Strip *p;
   int n;
@@ -43,11 +47,7 @@ Fold *f;
     exit(1);
   }
   p->f = f;
-#ifdef MPI
-  n = f->size;
-#else
   n = f->count;
-#endif
   p->d = (Height *)malloc( n * sizeof(Height) );
   if( p->d == NULL )
   {
@@ -56,10 +56,15 @@ Fold *f;
   }
   return(p);
 }
-/*}}}*/
-/*{{{  void free_strip(Strip *p) */
+
+/* }}} */
+/* {{{   void free_strip(Strip *p) */
+#ifdef ANSI
+void free_strip(Strip *p) 
+#else
 void free_strip (p)
 Strip *p;
+#endif
 {
   if( p->d )
   {
@@ -68,10 +73,14 @@ Strip *p;
   }
   free(p);
 }
-/*}}}*/
-/*{{{  Strip *double_strip(Strip s) */
+/* }}} */
+/* {{{   Strip *double_strip(Strip *s) */
+#ifdef ANSI
+Strip *double_strip(Strip *s) 
+#else
 Strip *double_strip (s)
 Strip *s;
+#endif
 {
   Strip *p;
   Height *a, *b;
@@ -79,9 +88,6 @@ Strip *s;
   int count;
 
   p = make_strip(s->f->parent);
-#ifdef MPI
-  redistribude data
-#else
   a = s->d;
   b = p->d;
   count = s->f->count;
@@ -94,14 +100,17 @@ Strip *s;
     b++;
   }
   *b = *a;
-#endif
   return(p);
 }
-/*}}}*/
-/*{{{  Strip *set_strip(Fold *f, Height value)*/
+/* }}} */
+/* {{{   Strip *set_strip(Fold *f, Height value)*/
+#ifdef ANSI
+Strip *set_strip(Fold *f, Height value)
+#else
 Strip *set_strip (f,value)
 Fold *f;
 Height value;
+#endif
 {
   int i;
   Strip *s;
@@ -110,11 +119,7 @@ Height value;
   
   s = make_strip(f);
   h = s->d;
-#ifdef MPI
-  count = f->size;
-#else
   count = f->count;
-#endif
   for( i=0 ; i < count ; i++)
   {
     *h = value;
@@ -122,29 +127,57 @@ Height value;
   }
   return(s);
 }
-/*}}}*/
-/*{{{  Strip *random_strip(Fold *f)*/
+
+/* }}} */
+/* {{{   Strip *random_strip(Fold *f)*/
+#ifdef ANSI
+Strip *random_strip(Fold *f)
+#else
 Strip *random_strip(f)
 Fold *f;
+#endif
 {
   Strip *result;
   int i, count;
   
   result=make_strip(f);
-#ifdef MPI
-  count=f->size;  /* this is a little wasteful as it also sets the halo */
-#else
   count = f->count;
-#endif
   for( i=0 ; i < count ; i++)
   {
     result->d[i] = f->p->mean + (f->scale * gaussian());
   }
   return(result);
 }
-/*}}}*/
+/* }}} */
 
-/*{{{  Fold *make_fold(Fold *parent,Parm *param, int levels, int stop, Length len) */
+/* {{{ void reset_fold(Fold *f) */
+#ifdef ANSI
+void reset_fold(Fold *f) 
+#else
+void reset_fold(f)
+Fold *f;
+#endif
+{
+  /*
+   * resets any cached values within the fold structure
+   * this should be called if the param struct is changed.
+   */
+  Length scale, midscale;
+  double root2;
+
+  root2=sqrt((double) 2.0 );
+  scale = pow((double) f->length, (double) (2.0 * f->p->fdim));
+  midscale = pow((((double) f->length)*root2), (double) (2.0 * f->p->fdim));
+  f->scale = scale;
+  f->midscale = midscale;
+
+  if( f->next ){
+    reset_fold(f->next);
+  }
+}
+
+/* }}} */
+/* {{{   Fold *make_fold(Fold *parent,Parm *param, int levels, int stop, Length len) */
 /*
  * Initialise the fold structures.
  * As everything else reads the parameters from their fold
@@ -161,16 +194,18 @@ Fold *f;
  *   len gets smaller as the level increases.
  * start, the starting height for a non-fractal start.
  */
+#ifdef ANSI
+Fold *make_fold(Fold *parent,Parm *param, int levels, int stop, Length length) 
+#else
 Fold *make_fold (parent,param,levels,stop,length)
 struct fold *parent;
 struct parm *param;
 int levels;
 int stop;
 Length length;
+#endif
 {
   Fold *p;
-  Length scale, midscale;
-  double root2;
   int i;
 
   if( (levels < stop) || (stop<0) )
@@ -185,26 +220,22 @@ Length length;
     fprintf(stderr,"make_fold: malloc failed\n");
     exit(1);
   }
-  root2=sqrt((double) 2.0 );
-  scale = pow((double) length, (double) (2.0 * param->fdim));
-  midscale = pow((((double) length)*root2), (double) (2.0 * param->fdim));
+  p->length=length;
   p->level = levels;
   p->count = (1 << levels) +1;
   p->stop = stop;
   p->state = START;
   p->save =NULL;
   p->p = param;
-  p->scale = scale;
-  p->midscale = midscale;
   for(i=0;i<NSTRIP;i++)
   {
     p->s[i] = NULL;
   }
   p->parent=parent;
-#ifdef MPI
-  /* work out the distribution at this level */
-  p->first = ( p->p-me == 0 );
-#endif
+
+  p->next = NULL; /* truncate recursion in reset */
+  reset_fold(p);
+
   if( levels > stop )
   {
     p->next = make_fold(p,param,(levels-1),stop,(2.0*length));
@@ -213,12 +244,24 @@ Length length;
   }
   return( p );
 }
-/*}}}*/
-/*{{{  void free_fold(Fold *f) */
+/* }}} */
+/* {{{   void free_fold(Fold *f) */
+#ifdef ANSI
+void free_fold(Fold *f) 
+#else
 void free_fold (f)
 Fold *f;
+#endif
 {
   int i;
+  if( f->next ){
+    free_fold(f->next);
+    f->next=NULL;
+  }
+  if( f->save ){
+    free_strip(f->save);
+    f->save=NULL;
+  }
   for(i=0;i<NSTRIP;i++)
   {
     if( f->s[i] != NULL )
@@ -230,11 +273,15 @@ Fold *f;
   free(f);
   return;
 }
-/*}}}*/
+/* }}} */
 
-/*{{{  Strip *next_strip(Fold *fold) */
+/* {{{   Strip *next_strip(Fold *fold) */
+#ifdef ANSI
+Strip *next_strip(Fold *fold) 
+#else
 Strip *next_strip (fold)
 Fold *fold;
+#endif
 {
   Strip *result=NULL;
   Strip *tmp;
@@ -244,9 +291,9 @@ Fold *fold;
 
   if( fold->level == fold->stop)
   {
-    /*{{{  generate values from scratch */
+    /* {{{   generate values from scratch */
     result=random_strip(fold);
-    /*}}}*/
+    /* }}} */
   }else{
   /*
    * There are two types of strip,
@@ -260,11 +307,11 @@ Fold *fold;
    */
     while( result == NULL )
     {
-      /*{{{iterate*/
+      /* {{{ iterate*/
       switch(fold->state)
       {
         case START:
-          /*{{{  perform an update. return first result*/
+          /* {{{   perform an update. return first result*/
           DB("S1",fold);
           t=fold->s;
           /* read in a new A strip at the start of the pipeline */
@@ -387,9 +434,9 @@ Fold *fold;
           t[0]=t[1]=NULL;
           fold->state = STORE;
           break;
-          /*}}}*/
+          /* }}} */
         case STORE:
-          /*{{{  return second value from previous update. */
+          /* {{{   return second value from previous update. */
           result = fold->save;
           fold->save=NULL;
           for(i=NSTRIP-1;i>1;i--)
@@ -399,13 +446,13 @@ Fold *fold;
           fold->s[0] = fold->s[1]=NULL;
           fold->state = START;
           break;
-          /*}}}*/
+          /* }}} */
         default:
           fprintf(stderr,"next_strip: invalid state level %d state %d\n",
                fold->level,fold->state);
           exit(3);
       }
-      /*}}}*/
+      /* }}} */
     }
   }
   iter = fold->level - fold->stop;
@@ -417,9 +464,12 @@ Fold *fold;
   }
   return(result);
 }
-/*}}}*/
+/* }}} */
 
-/*{{{void x_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+/* {{{ void x_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+#ifdef ANSI
+void x_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)
+#else
 void x_update(fold, scale, mix, a, b, c)
 Fold *fold;
 float scale;
@@ -427,6 +477,7 @@ float mix;
 Strip *a;
 Strip *b;
 Strip *c;
+#endif
 {
   int i;
   int count=fold->count;
@@ -447,7 +498,7 @@ Strip *c;
   rp=c->d;
 
   if( mix <= 0.0 ){
-    /*{{{random offset to average of new points*/
+    /* {{{ random offset to average of new points*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = 0.25 * ( lp[0] + rp[0] + lp[2] + rp[2])
@@ -456,9 +507,9 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else if( mix >= 1.0 ){
-    /*{{{random offset to old value*/
+    /* {{{ random offset to old value*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = mp[1]
@@ -467,9 +518,9 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else{
-    /*{{{mixed update*/
+    /* {{{ mixed update*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = (mix * mp[1]) + w * ( lp[0] + rp[0] + lp[2] + rp[2])
@@ -478,11 +529,14 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }
 }    
-/*}}}*/
-/*{{{void p_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+/* }}} */
+/* {{{ void p_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+#ifdef ANSI
+void p_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)
+#else
 void p_update(fold, scale, mix, a, b, c)
 Fold *fold;
 float scale;
@@ -490,6 +544,7 @@ float mix;
 Strip *a;
 Strip *b;
 Strip *c;
+#endif
 {
   int i;
   int count=fold->count;
@@ -515,7 +570,7 @@ Strip *c;
   rp=c->d;
 
   if( mix <= 0.0 ){
-    /*{{{random offset to average of new points*/
+    /* {{{ random offset to average of new points*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = 0.25 * ( lp[1] + rp[1] + mp[0] + mp[2] )
@@ -524,9 +579,9 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else if(mix >= 1.0){
-    /*{{{random offset to old values*/
+    /* {{{ random offset to old values*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = mp[1]
@@ -535,9 +590,9 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else{
-    /*{{{mixed update*/
+    /* {{{ mixed update*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = (mix * mp[1]) + w * ( lp[1] + rp[1] + mp[0] + mp[2] )
@@ -546,12 +601,15 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }
 }    
 
-/*}}}*/
-/*{{{void t_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+/* }}} */
+/* {{{ void t_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+#ifdef ANSI
+void t_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)
+#else
 void t_update(fold, scale, mix, a, b, c)
 Fold *fold;
 float scale;
@@ -559,6 +617,7 @@ float mix;
 Strip *a;
 Strip *b;
 Strip *c;
+#endif
 {
   int i;
   int count=fold->count;
@@ -581,7 +640,7 @@ Strip *c;
   rp=c->d;
 
   if( mix <= 0.0){
-    /*{{{random offset to average of new points*/
+    /* {{{ random offset to average of new points*/
     mp[0] = third * ( lp[0] + rp[0] + mp[1] )
             + (scale * gaussian());
     mp++;
@@ -597,9 +656,9 @@ Strip *c;
     }
     mp[1] = third * ( lp[1] + rp[1] + mp[0] )
           + (scale * gaussian());
-    /*}}}*/
+    /* }}} */
   }else if(mix >= 1.0){
-    /*{{{random offset to old values*/
+    /* {{{ random offset to old values*/
     for(i=0; i<count; i+=2)
     {
       mp[0] = mp[0]
@@ -608,9 +667,9 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else{
-    /*{{{mixed update*/
+    /* {{{ mixed update*/
     mp[0] = (mix * mp[0]) + we * ( lp[0] + rp[0] + mp[1] )
             + (scale * gaussian());
     mp++;
@@ -626,13 +685,16 @@ Strip *c;
     }
     mp[1] = (mix * mp[1]) + we * ( lp[1] + rp[1] + mp[0] )
           + (scale * gaussian());
-    /*}}}*/
+    /* }}} */
   }
 }    
 
 
-/*}}}*/
-/*{{{void v_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+/* }}} */
+/* {{{ void v_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+#ifdef ANSI
+void v_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)
+#else
 void v_update(fold, scale, mix, a, b, c)
 Fold *fold;
 float scale;
@@ -640,6 +702,7 @@ float mix;
 Strip *a;
 Strip *b;
 Strip *c;
+#endif
 {
   int i;
   int count=fold->count;
@@ -661,7 +724,7 @@ Strip *c;
   rp=c->d;
 
   if( mix <= 0.0){
-    /*{{{random offset of average of new points*/
+    /* {{{ random offset of average of new points*/
     mp[0] = 0.5 * ( lp[1] + rp[1] )
             + (scale * gaussian());
     mp++;
@@ -677,9 +740,9 @@ Strip *c;
     }
     mp[1] = 0.5 * ( lp[0] + rp[0] )
             + (scale * gaussian());
-    /*}}}*/
+    /* }}} */
   }else if(mix >= 1.0){
-    /*{{{random offset to old values*/
+    /* {{{ random offset to old values*/
     for(i=0; i<count; i+=2)
     {
       mp[0] = mp[0]
@@ -688,9 +751,9 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else{
-    /*{{{mixed update*/
+    /* {{{ mixed update*/
     mp[0] = (mix * mp[0]) + we * ( lp[1] + rp[1] )
             + (scale * gaussian());
     mp++;
@@ -706,17 +769,21 @@ Strip *c;
     }
     mp[1] = (mix * mp[1]) + we * ( lp[0] + rp[0] )
             + (scale * gaussian());
-    /*}}}*/
+    /* }}} */
   }
 }    
 
-/*}}}*/
-/*{{{void vside_update(Fold *fold,float scale, float mix, Strip *a)*/
+/* }}} */
+/* {{{ void vside_update(Fold *fold,float scale, float mix, Strip *a)*/
+#ifdef ANSI
+void vside_update(Fold *fold,float scale, float mix, Strip *a)
+#else
 void vside_update(fold, scale, mix, a)
 Fold *fold;
 float scale;
 float mix;
 Strip *a;
+#endif
 {
   int i;
   int count=fold->count;
@@ -731,38 +798,41 @@ Strip *a;
   mp=a->d;
 
   if( mix <= 0.0){
-    /*{{{random offset to average of new points*/
+    /* {{{ random offset to average of new points*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = 0.5 * ( mp[0] + mp[2] )
             + (scale * gaussian());
       mp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else if(mix >= 1.0){
-    /*{{{random offset to old values*/
+    /* {{{ random offset to old values*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = mp[1]
             + (scale * gaussian());
       mp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else{
-    /*{{{mixed update*/
+    /* {{{ mixed update*/
     for(i=0; i<count-2; i+=2)
     {
       mp[1] = (mix * mp[1]) + w * ( mp[0] + mp[2] )
             + (scale * gaussian());
       mp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }
 }    
 
 
-/*}}}*/
-/*{{{void hside_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+/* }}} */
+/* {{{ void hside_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)*/
+#ifdef ANSI
+void hside_update(Fold *fold,float scale, float mix, Strip *a, Strip *b, Strip *c)
+#else
 void hside_update(fold, scale, mix, a, b, c)
 Fold *fold;
 float scale;
@@ -770,6 +840,7 @@ float mix;
 Strip *a;
 Strip *b;
 Strip *c;
+#endif
 {
   int i;
   int count=fold->count;
@@ -790,7 +861,7 @@ Strip *c;
   rp=c->d;
 
   if( mix <= 0.0 ){
-    /*{{{random offset to average of new points*/
+    /* {{{ random offset to average of new points*/
     for(i=0; i<count; i+=2)
     {
       mp[0] = 0.5 * ( lp[0] + rp[0] )
@@ -799,9 +870,9 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else if(mix >= 1.0){
-    /*{{{random offset to old points*/
+    /* {{{ random offset to old points*/
     for(i=0; i<count; i+=2)
     {
       mp[0] = mp[0]
@@ -810,9 +881,9 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }else{
-    /*{{{mixed update*/
+    /* {{{ mixed update*/
     for(i=0; i<count; i+=2)
     {
       mp[0] = (mix * mp[0]) + w * ( lp[0] + rp[0] )
@@ -821,13 +892,13 @@ Strip *c;
       lp+=2;
       rp+=2;
     }
-    /*}}}*/
+    /* }}} */
   }
 }    
 
 
 
-/*}}}*/
+/* }}} */
 
 
 #ifdef DEBUG
